﻿using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Amqp;
using AmqpClient;
using Amqp.Framing;
using System.Threading;

namespace Test.AmqpClient
{
    [TestClass]
    public class AmqpClientTests
    {
        //Address address = new Address("amqp://guest:guest@localhost:5672");

        Address address = new Address("amqps://owner:epygvBczIgLTouTw%2BuhP2w3Esi3NG5hWpMiOVPEp0X8%3D@amqpnetlitetest.servicebus.windows.net");

        [TestInitialize]
        public void Initialize()
        {
            // uncomment the following to write frame traces
            Trace.TraceLevel = TraceLevel.Verbose | TraceLevel.Frame;
            Trace.TraceListener = (f, a) => System.Diagnostics.Trace.WriteLine(DateTime.Now.ToString("[hh:ss.fff]") + " " + string.Format(f, a));
        }

        [TestMethod]
        public void TestMethod_BasicSendReceive()
        {

            AmqpSender sender = new AmqpSender(address, "q1");

            for (int i = 0; i < 200; ++i)
            {
                Message message = new Message();
                message.Properties = new Properties() { GroupId = "abcdefg" };
                message.ApplicationProperties = new ApplicationProperties();
                message.ApplicationProperties["sn"] = i;
                sender.Send(message);
            }

            AmqpReceiver receiver = new AmqpReceiver(address, "q1");
            for (int i = 0; i < 200; ++i)
            {
                if (i % 50 == 0) receiver.SetCredit(50);
                Message message = receiver.Receive();
                Trace.WriteLine(TraceLevel.Information, "receive: {0}", message.ApplicationProperties["sn"]);
                receiver.Complete(message);
            }

            sender.Close();
            receiver.Close();
        }

        [TestMethod]
        public void TestMethod_OnMessage()
        {
            AmqpReceiver receiver = new AmqpReceiver(address, "q1");
            ManualResetEvent done = new ManualResetEvent(false);
            receiver.OnMessage((r, m) =>
            {
                Trace.WriteLine(TraceLevel.Information, "receive: {0}", m.ApplicationProperties["sn"]);
                r.Complete(m);
                if ((int)m.ApplicationProperties["sn"] == 199)
                {
                    done.Set();
                }
            }, 200);

            AmqpSender sender = new AmqpSender(address, "q1");
            for (int i = 0; i < 200; ++i)
            {
                Message message = new Message();
                message.Properties = new Properties() { GroupId = "abcdefg" };
                message.ApplicationProperties = new ApplicationProperties();
                message.ApplicationProperties["sn"] = i;
                sender.Send(message);
            }


            done.WaitOne(10000, true);

            sender.Close();
            receiver.Close();
        }

        [TestMethod]
        public void TestMethod_ReceiveWaiter()
        {
            
            AmqpReceiver receiver = new AmqpReceiver(address, "q1");
            Thread t = new Thread(() =>
            {
                receiver.SetCredit(1);
                Message message = receiver.Receive();
                Trace.WriteLine(TraceLevel.Information, "receive: {0}", message.Properties.MessageId);
                receiver.Complete(message);
            });

            t.Start();

            AmqpSender sender = new AmqpSender(address, "q1");
            Message msg = new Message() { Properties = new Properties() { MessageId = "123456" } };
            sender.Send(msg);

            t.Join(10000);

            sender.Close();
            receiver.Close();
        }
    }
}
