﻿using System.Collections;

namespace IoT.Device.CommLayer
{
    /// <summary>
    /// Command received delegate
    /// </summary>
    /// <param name="sender">Object sender</param>
    /// <param name="properties">Dictionary with message properties</param>
    public delegate bool CommandReceivedCallback(object sender, IDictionary properties);

    /// <summary>
    /// Interface for communication layer
    /// </summary>
    public interface ICommLayer
    {
        /// <summary>
        /// Start communication layer
        /// </summary>
        /// <param name="deviceId">Device ID</param>
        /// <param name="deviceKey">Device Key</param>
        /// <param name="hostUri">Base Host URI</param>
        /// <param name="ingressUri">Ingress URI (telemetry, notification)</param>
        /// <param name="egressUri">Egress URI (command)</param>
        /// <param name="cmdIngressUri">Command ingress URI (command reply)</param>
        void Start(string deviceId, string deviceKey, string hostUri, string egressUri, string ingressUri, string cmdIngressUri);

        /// <summary>
        /// Stop communication layer
        /// </summary>
        void Stop();

        /// <summary>
        /// Send telemetry data
        /// </summary>
        /// <param name="deviceId">Device ID</param>
        /// <param name="subject">Telemetry subject</param>
        /// <param name="id">Instance (sensor) id</param>
        /// <param name="value">Data value</param>
        void SendTelemetry(string deviceId, string subject, uint id, object value);

        /// <summary>
        /// Send a command reply
        /// </summary>
        /// <param name="deviceId">Device ID</param>
        /// <param name="gesture">Command gesture</param>
        /// <param name="relatesTo">Command related to reply</param>
        /// <param name="result">Result value</param>
        void SendCommandReply(string deviceId, string gesture, string relatesTo, object result);

        /// <summary>
        /// Event raised on command received
        /// </summary>
        event CommandReceivedCallback OnCommandReceived;
    }
}
