﻿using System;
using System.Threading;
#if (MF_FRAMEWORK_VERSION_V4_3)
using Microsoft.SPOT;
#endif
using IoT.Device.Sensors;
using IoT.Device.CommLayer;

namespace IoT.Device.App
{
    class Program
    {
        // device identification (ID and Key)
        private string deviceId = "<deviceId>";
        private string deviceKey = "<deviceKey>";
        private string hostUri = "<hostUri>";
        // ingress and egress URI for messages exchanges
        private string egressUri = "/out/t0001/Subscriptions/s0000";
        private string ingressUri = "/in/t0001";
        private string cmdIngressUri = "/in/t0001";

        // timer for sending telemetry data
        private Timer timer;

        // sensors
        private ITemperatureSensor temperatureSensor;
        private IHumiditySensor humiditySensor;
        private IPowerSensor powerSensor;

        // IoT Service for sending data and receiving commands
        private IoTService iotService;

        static void Main(string[] args)
        {
            (new Program()).Run();
        }

        private void Run()
        {
            // create sensor instances
            this.temperatureSensor = new DHT11();
            // NOTE : DHT11 is temperature and humidity sensor
            this.humiditySensor = (IHumiditySensor)this.temperatureSensor;
            this.powerSensor = new Power();

            // create IoTService instance with AMQP communication layer
            this.iotService = new IoTService(new AmqpCommLayer());

            this.iotService.Init(this.deviceId,
                this.deviceKey,
                this.hostUri,
                this.ingressUri,
                this.egressUri,
                this.cmdIngressUri);

            // start service
            this.iotService.Start();

            this.timer = new Timer(this.TimerCallback, null, 1000, Timeout.Infinite);

            Console.WriteLine("Press any Key to stop ...");
            Console.ReadLine();

            this.timer.Change(Timeout.Infinite, Timeout.Infinite);

            // stop service
            this.iotService.Stop();
        }

        private void TimerCallback(object state)
        {
            try
            {
                // read telemetry data ...
                double temperature = this.temperatureSensor.GetTemperature();
                double humidity = this.humiditySensor.GetHumidity();
                double power = this.powerSensor.GetPower();

                // ... and send them
                this.iotService.SendTelemetry(IoTService.SubjectType.Temperature, 0, temperature);
                this.iotService.SendTelemetry(IoTService.SubjectType.Humidity, 0, humidity);
                this.iotService.SendTelemetry(IoTService.SubjectType.Power, 0, power);
            }
            catch
            { }
        }
    }
}
