﻿using System;
using System.Net;
using System.Net.Sockets;
using System.Threading;
using Microsoft.SPOT;
using Microsoft.SPOT.Hardware;
using SecretLabs.NETMF.Hardware;
using SecretLabs.NETMF.Hardware.Netduino;
using IoT.Device;
using IoT.Device.Sensors;
using uPLibrary.Hardware;
using IoT.Device.CommLayer;

namespace TelemetryMqttNetduinoApp.NetMf43
{
    public class Program
    {
        // device identification (ID and Key)
        private string deviceId = "<deviceId>";
        private string deviceKey = "<deviceKey>";
        private string hostUri = "<hostUri>";
        // ingress and egress URI for messages exchanges
        private string egressUri = "/out/t0003/Subscriptions/s0000";
        private string ingressUri = "/in/t000B";
        private string cmdIngressUri = "/in/t000B";

        // timer for sending telemetry data
        private Timer timer;

        // sensors
        private ITemperatureSensor temperatureSensor;
        private IHumiditySensor humiditySensor;

        // IoT Service for sending data and receiving commands
        private IoTService iotService;

        // variables used to store current values 
        private double currentHumidity = double.MinValue;
        private double currentTemperature = double.MinValue;

        static void Main(string[] args)
        {
            (new Program()).Run();
        }

        private void Run()
        {
            // create sensor instances
            this.temperatureSensor = new SHT1X(Pins.GPIO_PIN_D0, Pins.GPIO_PIN_D1, SHT1X.Voltage.Vdd_3_5V);
            // NOTE : DHT11 is temperature and humidity sensor
            this.humiditySensor = (IHumiditySensor)this.temperatureSensor;

            // create IoTService instance with MQTT communication layer
            this.iotService = new IoTService(new MqttCommLayer());

            this.iotService.Init(this.deviceId,
                this.deviceKey,
                this.hostUri,
                this.ingressUri,
                this.egressUri,
                this.cmdIngressUri);

            // start service
            this.iotService.Start();

            this.timer = new Timer(this.TimerCallback, null, 1000, 10000);

            InputPort reset = new InputPort(Pins.ONBOARD_BTN, true, Port.ResistorMode.Disabled);

            while (!reset.Read()) { }

            this.timer.Change(Timeout.Infinite, Timeout.Infinite);

            // stop service
            this.iotService.Stop();
        }

        private void TimerCallback(object state)
        {
            try
            {
                // read telemetry data ...
                double temperature = this.temperatureSensor.GetTemperature();
                double humidity = this.humiditySensor.GetHumidity();

                temperature = System.Math.Round(temperature);
                humidity = System.Math.Round(humidity);

                Debug.Print("tmp = " + temperature + " hum = " + humidity);

                // ... and send them if changed
                if (temperature != this.currentTemperature)
                {
                    this.currentTemperature = temperature;
                    this.iotService.SendTelemetry(IoTService.SubjectType.Temperature, 0, temperature);
                }
                if (humidity != this.currentHumidity)
                {
                    this.currentHumidity = humidity;
                    this.iotService.SendTelemetry(IoTService.SubjectType.Humidity, 0, humidity);
                }
            }
            catch
            { }
        }
    }
}
