﻿using System;

namespace IoT.Device.Sensors
{
    /// <summary>
    /// DHT11 temperature and humidity sensor class
    /// </summary>
    public class DHT11 : ITemperatureSensor, IHumiditySensor
    {
        // avarage values to get random data
        private const double TEMPERATURE_AVG = 20;
        private const double HUMIDITY_AVG = 50;

        // random instance for generating data
        private Random rand;

        // last read value (-1 at startup)
        private double lastTemperature = -1;
        private double lastHumidity = -1;

        /// <summary>
        /// Constructor
        /// </summary>
        public DHT11()
        {
            // create random generator with millseconds as seed
            this.rand = new Random(DateTime.Now.Millisecond);
        }

        #region ITemperatureSensor ...

        public double GetTemperature()
        {
            if (this.lastTemperature < 0)
            {
                this.lastTemperature = TEMPERATURE_AVG + this.rand.NextDouble() - 0.5;
            }
            else
            {
                double change = (this.rand.NextDouble() - 0.5) / 10;
                this.lastTemperature += change;
            }

            return this.lastTemperature;
        }

        #endregion

        #region IHumiditySensor ...

        public double GetHumidity()
        {
            if (this.lastHumidity < 0)
            {
                this.lastHumidity = HUMIDITY_AVG + this.rand.NextDouble() - 0.5;
            }
            else
            {
                double change = (this.rand.NextDouble() - 0.5) / 10;
                this.lastHumidity += change;
            }

            return this.lastHumidity;
        }

        #endregion
    }
}
