﻿using System;

namespace IoT.Device.Sensors
{
    /// <summary>
    /// Power sensor class
    /// </summary>
    public class Power : IPowerSensor
    {
        // avarage value to get random data
        private const double POWER_AVG = 100;

        // random instance for generating data
        private Random rand;

        // last read value (-1 at startup)
        private double lastPower = -1;

        /// <summary>
        /// Constructor
        /// </summary>
        public Power()
        {
            // create random generator with millseconds as seed
            this.rand = new Random(DateTime.Now.Millisecond);
        }

        #region IPowerSensor ...

        public double GetPower()
        {
            if (this.lastPower < 0)
            {
                this.lastPower = POWER_AVG + this.rand.NextDouble() - 0.5;
            }
            else
            {
                double change = (this.rand.NextDouble() - 0.5) / 10;
                this.lastPower += change;
            }

            return this.lastPower;
        }

        #endregion
    }
}
