﻿using System;
using IoT.Device.CommLayer;
using IoT.Device;
using IoT.Device.Actuators;

namespace IoT.Device.WashingMachine.App
{
    class Program
    {
        // device identification (ID and Key)
        private string deviceId = "<deviceId>";
        private string deviceKey = "<deviceKey>";
        private string hostUri = "<hostUri>";
        // ingress and egress URI for messages exchanges
        private string egressUri = "/out/t0000/Subscriptions/s0000";
        private string ingressUri = "/in/t0000";
        private string cmdIngressUri = "/in/t0000";

        private IoTService iotService;

        private IWashing washing;

        static void Main(string[] args)
        {
            (new Program()).Run();
        }

        private void Run()
        {
            // create IoTService instance with AMQP communication layer
            this.iotService = new IoTService(new MqttCommLayer());

            this.iotService.Init(this.deviceId,
                this.deviceKey,
                this.hostUri,
                this.ingressUri,
                this.egressUri,
                this.cmdIngressUri);

            // register to washing commands
            this.iotService.OnWashingStart += iotService_OnWashingStart;
            this.iotService.OnWashingStop += iotService_OnWashingStop;

            // start service
            this.iotService.Start();

            // create actuator instances
            this.washing = new Washing();
            this.washing.OnWashingEnd += washing_OnWashingEnd;

            Console.WriteLine("Press any Key to stop ...");
            Console.ReadLine();

            // stop service
            this.iotService.Stop();
        }

        void iotService_OnWashingStop(string commandId)
        {
            if (this.washing.Stop())
                this.iotService.SendCommandReply("wsh", commandId, "ok");
            else
                this.iotService.SendCommandReply("wsh", commandId, "ko");
        }

        void iotService_OnWashingStart(string commandId, int program, int speed, int temperature)
        {
            if (this.washing.Start(program, speed, temperature))
                this.iotService.SendCommandReply("wsh", commandId, "ok");
            else
                this.iotService.SendCommandReply("wsh", commandId, "ko");
        }

        void washing_OnWashingEnd(object sender, EventArgs e)
        {
            this.iotService.SendTelemetry(IoTService.SubjectType.Notification, "ok");
        }
    }
}
